/*
 *     file: RegexPatternSetViewFilter.java
 *  package: org.eclipse.regex.tools.ui
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.regex.tools.ui;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;

/**
 * Implements a filter class that filters elements depending upon regular expression
 * patterns.
 * <p>
 * The class compares the element to the regular expressions in the set of patterns and
 * returns the result. The user should note that the more filter patterns that they use,
 * the slower the performance of the filter. It is advisable, instead, to try to
 * consolidate your patterns into a small set. For more information on pattern syntax the
 * user should see the javadoc for <code>java.util.regex.Pattern</code>.
 * </p>
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 * @see java.util.regex.Pattern
 */
public class RegexPatternSetViewFilter extends ViewerFilter {

    /**
     * The patterns used in the filter. This set holds instances of
     * <code>java.util.regex.Pattern</code>.
     * 
     * @see java.util.regex.Pattern
     */
    private final Set patterns = new HashSet();

    /** The mode to use for regex filtering. */
    private RegexFilterMode filterMode;

    /**
     * Creates a new <code>RegexPatternSetViewFilter</code>.
     */
    public RegexPatternSetViewFilter(final RegexFilterMode filterMode) {
        this.filterMode = filterMode;
    }

    /**
     * If the class name matches any of the patterns then it will be filtered <b>OUT </b>.
     * Only classes with names that dont match any of the set patterns will get through
     * the filter.
     * 
     * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public boolean select(final Viewer viewer, final Object parentElement, final Object element) {
        final String matchOperand = (String)element;

        // -- Count how many of the patterns match
        int matchCount = 0;
        for (final Iterator iter = this.patterns.iterator(); iter.hasNext();) {
            final Pattern pattern = (Pattern)iter.next();
            final Matcher matcher = pattern.matcher(matchOperand);
            if (matcher.matches()) {
                matchCount++;
            }
        }

        // -- Apply the match to the filter mode.
        if (this.filterMode == RegexFilterMode.FILTER_IN_ON_ALL) {
            return matchCount == this.patterns.size();
        } else if (this.filterMode == RegexFilterMode.FILTER_IN_ON_ANY) {
            return matchCount >= 1;
        } else if (this.filterMode == RegexFilterMode.FILTER_OUT_ON_ALL) {
            return !(matchCount == this.patterns.size());
        } else if (this.filterMode == RegexFilterMode.FILTER_OUT_ON_ANY) {
            return !(matchCount >= 1);
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_BRANCH);
    }

    /**
     * Getter for filterMode: The mode to use for regex filtering.
     * 
     * @return Returns the filterMode.
     */
    public RegexFilterMode getFilterMode() {
        return this.filterMode;
    }

    /**
     * Setter for filterMode: The mode to use for regex filtering.
     * 
     * @param filterMode The filterMode to set.
     */
    public void setFilterMode(RegexFilterMode filterMode) {
        this.filterMode = filterMode;
    }

    /**
     * Getter for patterns: The patterns used in the filter.
     * 
     * @return Returns the patterns in an unmodifiable set; to change the patterns use the
     *         setPatterns(Set) method.
     */
    public Set getPatterns() {
        return Collections.unmodifiableSet(this.patterns);
    }

    /**
     * Add all of the paterns in the given set to the patterns list.
     * 
     * @param patterns The patterns to add.
     * @throws IllegalArgumentException If there is an object in <code>patterns</code>
     *             that is not of type <code>java.util.regex.Pattern</code>.
     */
    public void addAllToPatterns(final Set patterns) {
        try {
            // Validate that the contents of the set are all Pattern objects.
            patterns.toArray(new Pattern [patterns.size()]);
        } catch (final ArrayStoreException ex) {
            throw new IllegalArgumentException(I18N.ERR_INVALID_ELEMENT + ex.getMessage());
        }
        this.patterns.addAll(patterns);
    }

    /**
     * Add the given pattern.
     * 
     * @param pattern The pattern to add.
     */
    public void addPattern(final Pattern pattern) {
        this.patterns.add(pattern);
    }

    /**
     * Remove all patterns in the given set.
     * 
     * @param pattern The set of patterns to remove.
     * @throws IllegalArgumentException If there is an object in <code>patterns</code>
     *             that is not of type <code>java.util.regex.Pattern</code>.
     */
    public void removeAllPatterns(final Set patterns) {
        try {
            // Validate that the contents of the set are all Pattern objects.
            patterns.toArray(new Pattern [patterns.size()]);
        } catch (final ArrayStoreException ex) {
            throw new IllegalArgumentException(I18N.ERR_INVALID_ELEMENT + ex.getMessage());
        }
        this.patterns.removeAll(patterns);
    }

    /**
     * Remove the given pattern.
     * 
     * @param pattern The pattern to remove.
     */
    public void removePattern(final Pattern pattern) {
        this.patterns.remove(pattern);
    }

    /**
     * Clear the patterns so that there are no more patterns.
     */
    public void clearPatterns() {
        this.patterns.clear();
    }
}

/* ########## End of File ########## */