/*
 *     file: RegexPatternEditor.java
 *  package: org.eclipse.regex.tools.ui
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.regex.tools.ui;

import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.regex.tools.plugin.RegexToolsPlugin;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

/**
 * A custom control for editing regular expression patterns with support for validating
 * the entered regular expressions.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public class RegexPatternEditor extends Composite implements SelectionListener {

    /** A cached image for the validate button. */
    private final Image imgValidate = RegexToolsPlugin.getPluginImage("icons/validate.gif").createImage(); //$NON-NLS-1$

    /** The text entry used for the pattern. */
    private final Text txtPattern;

    /** The checkbox for case insensitivity. */
    private final Button chkCaseInsensitive;

    /** The checkbox for canonical equivalence. */
    private final Button chkCanonEq;

    /** The checkbox for comments mode. */
    private final Button chkComments;

    /** The checkbox for dotall mode. */
    private final Button chkDotall;

    /** The checkbox for multiline mode. */
    private final Button chkMultiline;

    /** The checkbox for Unicode case folding mode. */
    private final Button chkUnicodeCaseFolding;

    /** The checkbox for Unix lines mode. */
    private final Button chkUnixLines;

    /** The button for validating the expression. */
    private final Button btnValidate;

    /** The button for setting defaults. */
    private final Button btnDefaults;

    /**
     * Creates a new <code>RegexPatternEditor</code> with the given parameters.
     * 
     * @param parent The parent composite.
     */
    public RegexPatternEditor(final Composite parent, final int style) {
        super(parent, style & ~SWT.NO_BACKGROUND & ~SWT.V_SCROLL & ~SWT.H_SCROLL);

        setLayout(new GridLayout(2, false));

        { // A Composite containing the text field and validate button.
            final Composite pvArea = new Composite(this, SWT.NONE);
            final GridLayout pvAreaLayout = new GridLayout(2, false);
            pvAreaLayout.marginHeight = 0;
            pvAreaLayout.marginWidth = 0;
            pvArea.setLayout(pvAreaLayout);

            { // -- The text entry for the pattern
                this.txtPattern = new Text(pvArea, SWT.BORDER);
                this.txtPattern.setToolTipText(I18N.TOOLTIP_PATTERN_FIELD);
                final GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
                this.txtPattern.setLayoutData(gridData);
            }

            { // The validate button.
                this.btnValidate = new Button(pvArea, SWT.PUSH);
                this.btnValidate.setImage(this.imgValidate);
                this.btnValidate.setToolTipText(I18N.TOOLTIP_VALIDATE_BTN);
                this.btnValidate.addSelectionListener(this);
                final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_END);
                this.btnValidate.setLayoutData(gridData);
            }

            // grid data for pvArea
            final GridData gridData = new GridData(GridData.FILL_BOTH);
            gridData.horizontalSpan = 2;
            pvArea.setLayoutData(gridData);
        }

        { // The checkbox for case insesitivity.
            this.chkCaseInsensitive = new Button(this, SWT.CHECK);
            this.chkCaseInsensitive.setText(I18N.CHK_CASE_INSENSITIVE);
            this.chkCaseInsensitive.setToolTipText(I18N.TOOLTIP_CASE_INSENSITIVE);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.chkCaseInsensitive.setLayoutData(gridData);
        }

        { // The checkbox for canonical equivalence
            this.chkCanonEq = new Button(this, SWT.CHECK);
            this.chkCanonEq.setText(I18N.CHK_CANON_EQ);
            this.chkCanonEq.setToolTipText(I18N.TOOLTIP_CANON_EQ);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.chkCanonEq.setLayoutData(gridData);
        }

        { // The checkbox for comments mode.
            this.chkComments = new Button(this, SWT.CHECK);
            this.chkComments.setText(I18N.CHK_COMMENTS);
            this.chkComments.setToolTipText(I18N.TOOLTIP_COMMENTS);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.chkComments.setLayoutData(gridData);
        }

        { // The checkbox for dotall mode.
            this.chkDotall = new Button(this, SWT.CHECK);
            this.chkDotall.setText(I18N.CHK_DOTALL);
            this.chkDotall.setToolTipText(I18N.TOOLTIP_DOTALL);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.chkDotall.setLayoutData(gridData);
        }

        { // The checkbox for multiline mode.
            this.chkMultiline = new Button(this, SWT.CHECK);
            this.chkMultiline.setText(I18N.CHK_MULTILINE);
            this.chkMultiline.setToolTipText(I18N.TOOLTIP_MULTILINE);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.chkMultiline.setLayoutData(gridData);
        }

        { // The checkbox for Unicode case folding.
            this.chkUnicodeCaseFolding = new Button(this, SWT.CHECK);
            this.chkUnicodeCaseFolding.setText(I18N.CHK_UNICODE_CASE);
            this.chkUnicodeCaseFolding.setToolTipText(I18N.TOOLTIP_UNICODE_CASE);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.chkUnicodeCaseFolding.setLayoutData(gridData);
        }

        { // The checkbox for Unix lines.
            this.chkUnixLines = new Button(this, SWT.CHECK);
            this.chkUnixLines.setText(I18N.CHK_UNIX_LINES);
            this.chkUnixLines.setToolTipText(I18N.TOOLTIP_UNIX_LINES);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.chkUnixLines.setLayoutData(gridData);
        }

        { // The checkbox for Unix lines.
            this.btnDefaults = new Button(this, SWT.PUSH);
            this.btnDefaults.setText(I18N.BTN_DEFAULTS);
            this.btnDefaults.addSelectionListener(this);
            this.btnDefaults.setToolTipText(I18N.TOOLTIP_DEFAULTS);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
            this.btnDefaults.setLayoutData(gridData);
        }

        pack();
        loadDefaults();
    }

    /**
     * Sets the checkboxes and pattern to the default valuse.
     */
    public void loadDefaults() {
        // set the flag checkboxes with the default pattern options
        final Pattern initPtn = Pattern.compile(""); //$NON-NLS-1$
        setRegexPattern(initPtn);
    }

    /**
     * Gets the regular expression based on the data in the control.
     * 
     * @return A regular expression pattern based on the data in the control or null if
     *         the regular expression currently in the control cant be validated.
     */
    protected Pattern getRegexPattern() {
        // -- process the flags by examining which checkboxes are selected.
        int flags = 0;
        if (this.chkCaseInsensitive.getSelection()) {
            flags |= Pattern.CASE_INSENSITIVE;
        }
        if (this.chkCanonEq.getSelection()) {
            flags |= Pattern.CANON_EQ;
        }
        if (this.chkComments.getSelection()) {
            flags |= Pattern.COMMENTS;
        }
        if (this.chkDotall.getSelection()) {
            flags |= Pattern.DOTALL;
        }
        if (this.chkMultiline.getSelection()) {
            flags |= Pattern.MULTILINE;
        }
        if (this.chkUnicodeCaseFolding.getSelection()) {
            flags |= Pattern.UNICODE_CASE;
        }
        if (this.chkUnixLines.getSelection()) {
            flags |= Pattern.UNIX_LINES;
        }

        // -- Compile the patern and return it (or throw in compilation)
        try {
            return Pattern.compile(this.txtPattern.getText(), flags);
        } catch (final PatternSyntaxException ex) {
            MessageDialog.openError(this.btnValidate.getShell(), I18N.TITLE_DLG_REGEX_VALID,
                                    ex.getLocalizedMessage());
            return null;
        }
    }

    /**
     * Sets the contols of the editor to the values in the given pattern.
     * 
     * @param pattern The pattern to use.
     */
    protected void setRegexPattern(final Pattern pattern) {
        this.txtPattern.setText(pattern.pattern());

        // -- process the pattern flags
        final int flags = pattern.flags();
        if (0 != (flags & Pattern.CASE_INSENSITIVE)) {
            this.chkCaseInsensitive.setSelection(true);
        } else {
            this.chkCaseInsensitive.setSelection(false);
        }
        if (0 != (flags & Pattern.CANON_EQ)) {
            this.chkCanonEq.setSelection(true);
        } else {
            this.chkCanonEq.setSelection(false);
        }
        if (0 != (flags & Pattern.COMMENTS)) {
            this.chkComments.setSelection(true);
        } else {
            this.chkComments.setSelection(false);
        }
        if (0 != (flags & Pattern.DOTALL)) {
            this.chkDotall.setSelection(true);
        } else {
            this.chkDotall.setSelection(false);
        }
        if (0 != (flags & Pattern.MULTILINE)) {
            this.chkMultiline.setSelection(true);
        } else {
            this.chkMultiline.setSelection(false);
        }
        if (0 != (flags & Pattern.UNICODE_CASE)) {
            this.chkUnicodeCaseFolding.setSelection(true);
        } else {
            this.chkUnicodeCaseFolding.setSelection(false);
        }
        if (0 != (flags & Pattern.UNIX_LINES)) {
            this.chkUnixLines.setSelection(true);
        } else {
            this.chkUnixLines.setSelection(false);
        }
    }

    /**
     * @see org.eclipse.swt.widgets.Widget#dispose()
     */
    public void dispose() {
        this.imgValidate.dispose();
        super.dispose();
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#setEnabled(boolean,
     *      org.eclipse.swt.widgets.Composite)
     */
    public void setEnabled(final boolean enabled) {
        this.btnValidate.setEnabled(enabled);
        this.chkCaseInsensitive.setEnabled(enabled);
        this.chkCanonEq.setEnabled(enabled);
        this.chkComments.setEnabled(enabled);
        this.chkDotall.setEnabled(enabled);
        this.chkMultiline.setEnabled(enabled);
        this.chkUnicodeCaseFolding.setEnabled(enabled);
        this.chkUnixLines.setEnabled(enabled);
        this.txtPattern.setEnabled(enabled);
    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetDefaultSelected(final SelectionEvent event) {
        // ignore
    }

    /**
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetSelected(final SelectionEvent event) {
        final Object source = event.getSource();
        if (source == this.btnValidate) {
            final Pattern pattern = getRegexPattern();
            if (null != pattern) {
                MessageDialog.openInformation(this.btnValidate.getShell(),
                                              I18N.TITLE_DLG_REGEX_VALID, I18N.MXG_REGEX_VALID);
            }
        } else if (source == this.btnDefaults) {
            loadDefaults();
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_BRANCH);
    }

    /**
     * @see org.eclipse.swt.widgets.Control#setFocus()
     */
    public boolean setFocus() {
        return this.txtPattern.setFocus();
    }
}

/* ########## End of File ########## */