/*
 *     file: RegexPatternFieldEditor.java
 *  package: org.eclipse.regex.tools.ui
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.regex.tools.ui;

import java.util.StringTokenizer;
import java.util.regex.Pattern;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;

/**
 * A field editor that changes manipulates a regular expression pattern.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public class RegexPatternFieldEditor extends FieldEditor {

    /** A character used to separate the pattern from the flags in the encoded regexp. */
    private final static char FLAG_SEP = (char)1;

    /** The regex pattern editor. */
    private RegexPatternEditor rpePattern;

    /** The label for the editor. */
    private Label lblTitle = null;

    /**
     * Creates a new <code>RegexPatternFieldEditor</code> with the given parameters.
     * 
     * @param preferenceName The name of the preference to edit.
     * @param labelText The label text to use for the editor; if null is used then no text
     *            will be displayed on the border.
     * @param parent The parent composite of the editor.
     */
    public RegexPatternFieldEditor(final String preferenceName, final String labelText,
                                   final Composite parent) {
        super(preferenceName, labelText, parent);
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#adjustForNumColumns(int)
     */
    protected void adjustForNumColumns(final int numColumns) {
        ((GridData)this.rpePattern.getLayoutData()).horizontalSpan = numColumns;
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#doFillIntoGrid(org.eclipse.swt.widgets.Composite,
     *      int)
     */
    protected void doFillIntoGrid(final Composite parent, final int numColumns) {
        final Group group = new Group(parent, SWT.NONE);
        group.setText(getLabelText());
        group.setLayout(new GridLayout(1, true));

        { // The patern editor
            this.rpePattern = new RegexPatternEditor(group, SWT.NONE);
            final GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
            gridData.horizontalSpan = numColumns;
            this.rpePattern.setLayoutData(gridData);
        }
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#getLabelControl()
     */
    protected Label getLabelControl() {
        return this.lblTitle;
    }

    /**
     * Encode the pattern given for storage. This method is public so that it may be used
     * by plugins to set up defaults.
     * 
     * @param pattern The pattern to encode.
     * @return The encoded string.
     */
    public final static String encodePattern(final Pattern pattern) {
        // Implementation Notes: Store the pattern with the flags and the pattern in a
        // string that is separated by the FLAG_SEP character.
        final StringBuffer encodeBuffer = new StringBuffer(256);
        encodeBuffer.append(pattern.flags());
        encodeBuffer.append(FLAG_SEP);
        encodeBuffer.append(pattern);
        return encodeBuffer.toString();
    }

    /**
     * Decode the encoded pattern and create a new Pattern object. This method is public
     * so that it may be used by plugins to set up defaults.
     * 
     * @param encoded The pattern string to decode.
     * @return The pattern result of the decode.
     */
    public final static Pattern decodePattern(final String encoded) {
        // Implementation Notes: Please see encodePattern for information on encoding
        // format.
        final StringTokenizer tok = new StringTokenizer(encoded, Character.toString(FLAG_SEP),
                                                        false);
        final int flags = Integer.parseInt(tok.nextToken()); // First token is the flags.
        final String patternString = tok.nextToken(); // Second token is the pattern
        final Pattern pattern = Pattern.compile(patternString, flags);
        return pattern;
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#doLoad()
     */
    protected void doLoad() {
        final String encoded = getPreferenceStore().getString(getPreferenceName());
        this.rpePattern.setRegexPattern(RegexPatternFieldEditor.decodePattern(encoded));
    }

    /**
     * Note that if the pattern isnt valid, nothing will be stored.
     * 
     * @see org.eclipse.jface.preference.FieldEditor#doStore()
     */
    protected void doStore() {
        final Pattern pattern = this.rpePattern.getRegexPattern();
        getPreferenceStore().setValue(this.getPreferenceName(),
                                      RegexPatternFieldEditor.encodePattern(pattern));
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#doLoadDefault()
     */
    protected void doLoadDefault() {
        // Creates a pattern just to get the default flags out of the
        // pattern and then sets it in the controls. The default
        // pattern matches nothing.
        final Pattern defaultPattern = Pattern.compile(""); //$NON-NLS-1$
        this.rpePattern.setRegexPattern(defaultPattern);
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#dispose()
     */
    public void dispose() {
        super.dispose();
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#isValid()
     */
    public boolean isValid() {
        boolean result = (null != this.rpePattern.getRegexPattern());
        return result && super.isValid();
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#getNumberOfControls()
     */
    public int getNumberOfControls() {
        return 1;
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#setEnabled(boolean,
     *      org.eclipse.swt.widgets.Composite)
     */
    public void setEnabled(final boolean enabled, final Composite parent) {
        this.rpePattern.setEnabled(enabled);
        super.setEnabled(enabled, parent);
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#setFocus()
     */
    public void setFocus() {
        this.rpePattern.setFocus();
    }

    /**
     * @see org.eclipse.jface.preference.FieldEditor#setLabelText(java.lang.String)
     */
    public void setLabelText(String text) {
        this.lblTitle.setText(text);
    }
}

/* ########## End of File ########## */