/*
 *     file: RegexPatternLabelProvider.java
 *  package: org.eclipse.regex.tools.ui
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.regex.tools.ui;

import java.util.regex.Pattern;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

/**
 * Provides a simple label for a Regular expression pattern in a List or table.
 * <p>
 * The class selects characters to represent the flags set on the pattern and then appends
 * the spacer string and then pattern itself. Note that these characters are comlementary
 * of each other where the uppercase characters indicate that the feature is on and the
 * lower case case characters indicate that the feature is off.
 * <p>
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public class RegexPatternLabelProvider extends LabelProvider implements ITableLabelProvider {

    /** Spacer to use between flags and patterns in displaying regex patterns in lists. */
    public final static String SPACER_FLAG_PATTERN = "\u001A"; //$NON-NLS-1$

    /** Character Flag for case insensitivity on. */
    public final static char FLAG_CASE_INSENSITIVE_ON = 'I';

    /** Character Flag for case insensitivity off. */
    public final static char FLAG_CASE_INSENSITIVE_OFF = Character.toLowerCase(FLAG_CASE_INSENSITIVE_ON);

    /** Character Flag for canonical equivalence on. */
    public final static char FLAG_CANON_EQ_ON = 'E';

    /** Character Flag for canonical equivalence off. */
    public final static char FLAG_CANON_EQ_OFF = Character.toLowerCase(FLAG_CANON_EQ_ON);

    /** Character Flag for whitespace and comments on. */
    public final static char FLAG_COMMENTS_ON = 'C';

    /** Character Flag for whitespace and comments off. */
    public final static char FLAG_COMMENTS_OFF = Character.toLowerCase(FLAG_COMMENTS_ON);

    /** Character Flag for dotall mode on. */
    public final static char FLAG_DOTALL_ON = 'D';

    /** Character Flag for dotall mode off. */
    public final static char FLAG_DOTALL_OFF = Character.toLowerCase(FLAG_DOTALL_ON);

    /** Character Flag for multiline mode on. */
    public final static char FLAG_MULTILINE_ON = 'M';

    /** Character Flag for multiline mode off. */
    public final static char FLAG_MULTILINE_OFF = Character.toLowerCase(FLAG_MULTILINE_ON);

    /** Character Flag for Unicode aware case folding on. */
    public final static char FLAG_UNICODE_CASE_FOLDING_ON = 'F';

    /** Character Flag for Unicode aware case folding off. */
    public final static char FLAG_UNICODE_CASE_FOLDING_OFF = Character.toLowerCase(FLAG_UNICODE_CASE_FOLDING_ON);

    /** Character Flag for Unix lines mode on. */
    public final static char FLAG_UNIX_LINES_ON = 'U';

    /** Character Flag for nUnix lines mode off. */
    public final static char FLAG_UNIX_LINES_OFF = Character.toLowerCase(FLAG_UNIX_LINES_ON);

    /**
     * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object,
     *      int)
     */
    public Image getColumnImage(final Object element, final int columnIndex) {
        return null;
    }

    private String getFlagsString(final Pattern pattern) {
        final StringBuffer buffer = new StringBuffer(7);

        // -- parse the flags
        if (0 != (pattern.flags() & Pattern.CANON_EQ)) {
            buffer.append(FLAG_CANON_EQ_ON);
        } else {
            buffer.append(FLAG_CANON_EQ_OFF);
        }

        if (0 != (pattern.flags() & Pattern.CASE_INSENSITIVE)) {
            buffer.append(FLAG_CASE_INSENSITIVE_ON);
        } else {
            buffer.append(FLAG_CASE_INSENSITIVE_OFF);
        }

        if (0 != (pattern.flags() & Pattern.COMMENTS)) {
            buffer.append(FLAG_COMMENTS_ON);
        } else {
            buffer.append(FLAG_COMMENTS_OFF);
        }

        if (0 != (pattern.flags() & Pattern.DOTALL)) {
            buffer.append(FLAG_DOTALL_ON);
        } else {
            buffer.append(FLAG_DOTALL_OFF);
        }

        if (0 != (pattern.flags() & Pattern.MULTILINE)) {
            buffer.append(FLAG_MULTILINE_ON);
        } else {
            buffer.append(FLAG_MULTILINE_OFF);
        }

        if (0 != (pattern.flags() & Pattern.UNICODE_CASE)) {
            buffer.append(FLAG_UNICODE_CASE_FOLDING_ON);
        } else {
            buffer.append(FLAG_UNICODE_CASE_FOLDING_OFF);
        }

        if (0 != (pattern.flags() & Pattern.UNIX_LINES)) {
            buffer.append(FLAG_UNIX_LINES_ON);
        } else {
            buffer.append(FLAG_UNIX_LINES_OFF);
        }
        return buffer.toString();
    }

    /**
     * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object,
     *      int)
     */
    public String getColumnText(final Object element, final int columnIndex) {
        final Pattern pattern = (Pattern)element;

        if (0 == columnIndex) {
            return getFlagsString(pattern);
        } else if (1 == columnIndex) {
            return pattern.pattern();
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_BRANCH);
    }

    /**
     * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
     */
    public String getText(final Object element) {
        final Pattern pattern = (Pattern)element;
        final StringBuffer buffer = new StringBuffer(100);

        buffer.append(getFlagsString(pattern));
        buffer.append(SPACER_FLAG_PATTERN);
        buffer.append(pattern.pattern());

        return buffer.toString();
    }
}

/* ########## End of File ########## */