/*
 *     file: ObjectCategory.java
 *  package: org.eclipse.swt.sleak.core
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.swt.sleak.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Region;
import org.eclipse.swt.graphics.TextLayout;

/**
 * A category of resource types.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public final class ObjectCategory implements Comparable {

    /** A registry of all of the categories. */
    private final static Map LOOKUP_TABLE = new HashMap();

    /** A category for <code>Color</code>s. */
    public final static ObjectCategory COLOR = new ObjectCategory("COLOR", Color.class); //$NON-NLS-1$

    /** A category for <code>Cursor</code>s. */
    public final static ObjectCategory CURSOR = new ObjectCategory("CURSOR", Cursor.class); //$NON-NLS-1$

    /** A category for <code>Font</code>s. */
    public final static ObjectCategory FONT = new ObjectCategory("FONT", Font.class); //$NON-NLS-1$

    /** A category for Graphic Contexts (<code>GC</code> s). */
    public final static ObjectCategory GC = new ObjectCategory("GC", GC.class); //$NON-NLS-1$

    /** A category for <code>Image</code>s. */
    public final static ObjectCategory IMAGE = new ObjectCategory("IMAGE", Image.class); //$NON-NLS-1$

    /** A category for <code>Region</code>s. */
    public final static ObjectCategory REGION = new ObjectCategory("REGION", Region.class); //$NON-NLS-1$

    /** A category for <code>TextLayouts</code>. */
    public final static ObjectCategory TEXT_LAYOUT = new ObjectCategory(
                                                                        "TEXT_LAYOUT", TextLayout.class); //$NON-NLS-1$

    /** The target class of the ObjectCategory. */
    private final Class targetClass;

    /** The name of the ObjectCategory. */
    private final String name;

    /**
     * Creates a new <code>ObjectCategory</code> with the given parameters.
     * 
     * @param name The name for the category.
     * @param targetClass The target class of the category.
     */
    private ObjectCategory(final String name, final Class targetClass) {
        this.name = name;
        this.targetClass = targetClass;
        ObjectCategory.LOOKUP_TABLE.put(name, this);
    }

    /**
     * Get a collection of all categories.
     * 
     * @return A collection of all categories.
     */
    public static List getAllCategories() {
        return Collections.unmodifiableList(new ArrayList(ObjectCategory.LOOKUP_TABLE.values()));
    }

    /**
     * Getter for name: The name of the ObjectCategory.
     * 
     * @return Returns the name.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Getter for targetClass: The target class of the ObjectCategory.
     * 
     * @return Returns the targetClass.
     */
    public Class getTargetClass() {
        return this.targetClass;
    }

    /**
     * Finds the correct category for the given object.
     * 
     * @param object The object to categorize.
     * @return The category for that object.
     */
    public static ObjectCategory categorize(final Object object) {
        for (final Iterator iter = LOOKUP_TABLE.values().iterator(); iter.hasNext();) {
            final ObjectCategory element = (ObjectCategory)iter.next();
            if (element.getTargetClass().isInstance(object)) {
                return element;
            }
        }
        throw new IllegalArgumentException(object.getClass().getName()); // not found
    }

    /**
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(final Object obj) {
        if (!(obj instanceof ObjectCategory)) {
            return -1;
        }
        return getName().compareTo(((ObjectCategory)obj).getName());
    }
}

/* ########## End of File ########## */