/*
 *     file: CacheContentProvider.java
 *  package: org.eclipse.swt.sleak.plugin.views
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.swt.sleak.plugin.views;

import java.util.Iterator;
import java.util.List;
import java.util.SortedSet;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.sleak.core.IDeviceDataCache;
import org.eclipse.swt.sleak.core.IObjectDeviceData;
import org.eclipse.swt.sleak.core.ISLeak;
import org.eclipse.swt.sleak.core.ISLeakEventListener;
import org.eclipse.swt.sleak.core.ObjectCategory;
import org.eclipse.swt.sleak.core.SLeakEvent;

/**
 * Prvides content from an <code>ISLeak</code> cache system to a SWT tree.
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public class CacheContentProvider implements ITreeContentProvider, ISLeakEventListener {

    /** The viewer for which content is being provided. */
    private TreeViewer viewer = null;

    /** The S-Leak being used. */
    private ISLeak sleak = null;

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(final Object parentElement) {
        if (parentElement instanceof IDeviceDataCache) {
            final IDeviceDataCache cache = (IDeviceDataCache)parentElement;
            final List categories = ObjectCategory.getAllCategories();
            // map the catecories and caches together.
            CategoryAndCache[] result = new CategoryAndCache [categories.size()];
            int idx = 0;
            for (final Iterator iter = categories.iterator(); iter.hasNext(); idx++) {
                final ObjectCategory category = (ObjectCategory)iter.next();
                result[idx] = new CategoryAndCache(category, cache);
            }
            return result;
        } else if (parentElement instanceof CategoryAndCache) {
            final CategoryAndCache cac = (CategoryAndCache)parentElement;
            final SortedSet dataItems = cac.cache.getCategory(cac.category);
            return (IObjectDeviceData[])dataItems.toArray(new IObjectDeviceData [dataItems.size()]);
        } else if (parentElement instanceof IObjectDeviceData) {
            return null;
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_ELEMENT + parentElement.getClass().getName());
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
     */
    public Object getParent(final Object element) {
        if (element instanceof IDeviceDataCache) {
            return this.sleak;
        } else if (element instanceof CategoryAndCache) {
            return ((CategoryAndCache)element).cache;
        } else if (element instanceof IObjectDeviceData) {
            return null;
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_ELEMENT + element.getClass().getName());
    }

    /**
     * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(final Object element) {
        if (element instanceof IDeviceDataCache) {
            return true;
        } else if (element instanceof CategoryAndCache) {
            final CategoryAndCache cac = (CategoryAndCache)element;
            return !cac.cache.getCategory(cac.category).isEmpty();
        } else if (element instanceof IObjectDeviceData) {
            return false;
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_ELEMENT + element.getClass().getName());
    }

    /**
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(final Object inputElement) {
        if (inputElement instanceof ISLeak) {
            final ISLeak sleak = (ISLeak)inputElement;
            return sleak.getCaches().toArray(new Object [sleak.getCaches().size()]);
        }
        throw new AssertionError(I18N.ERR_UNEXPECTED_ELEMENT + inputElement.getClass().getName());
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose() {
        // nothing needed here.
    }

    /**
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public void inputChanged(final Viewer viewer, final Object oldInput, final Object newInput) {
        this.viewer = (TreeViewer)viewer;
        this.sleak = (ISLeak)newInput;
        if (oldInput != null) {
            ((ISLeak)oldInput).removeSLeakListener(this);
        }
        if (newInput != null) {
            ((ISLeak)newInput).addSLeakListener(this);
        }
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeakEventListener#doCachesUpdated(org.eclipse.swt.sleak.core.SLeakEvent)
     */
    public void doCachesUpdated(SLeakEvent event) {
        try {
            if (this.viewer != null) {
                this.viewer.refresh();
            }
        } catch (final Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * @see org.eclipse.swt.sleak.core.ISLeakEventListener#doMonitorStateChanged(org.eclipse.swt.sleak.core.SLeakEvent)
     */
    public void doMonitorStateChanged(final SLeakEvent event) {
        // ignore
    }
}

/* ########## End of File ########## */