/*
 *     file: StackTraceFilter.java
 *  package: org.eclipse.swt.sleak.plugin.views
 *
 * This program and the accompanying materials are made available under 
 * the terms of the Common Public License v1.0 which accompanies this 
 * distribution, and is available at the following URL: 
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Copyright(c) 2003-2005 by the authors indicated in the @author tags.
 * All Rights are Reserved by the various authors.
 *
 ########## DO NOT EDIT ABOVE THIS LINE ########## */

package org.eclipse.swt.sleak.plugin.views;

import java.util.SortedSet;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.regex.tools.ui.RegexFilterMode;
import org.eclipse.regex.tools.ui.RegexPatternSetFieldEditor;
import org.eclipse.regex.tools.ui.RegexPatternSetViewFilter;
import org.eclipse.swt.sleak.plugin.SLeakPlugin;
import org.eclipse.swt.sleak.plugin.preference.SLeakPreferenceConstants;

/**
 * <p>
 * Implements a filter class that filters stack trace elements depending upon regular
 * expression patterns for class names that can be altered by the user.
 * </p>
 * <p>
 * The user should know that the patterns used in this class correspond to standard Perl
 * regular expression patters. For more information the user should see the javadoc for
 * <code>java.util.regex.Pattern</code>.
 * </p>
 * <p>
 * If the class name matches any of the patterns set in the filter then it will be
 * filtered <b>OUT </b>. Only classes with names that dont match any of the set patterns
 * will get through the filter.
 * </p>
 * 
 * @author <a href=mailto:kraythe@arcor.de>Robert Simmons Jr. (Kraythe) </a>
 */
public class StackTraceFilter extends RegexPatternSetViewFilter implements IPropertyChangeListener {

    /** Holds a reference to the viewer being filtered. */
    private final Viewer viewer;

    /**
     * Creates a new <code>StackTraceFilter</code>.
     * 
     * @param viewer The viewer being filtered.
     */
    public StackTraceFilter(final Viewer viewer) {
        super(RegexFilterMode.FILTER_OUT_ON_ANY);
        this.viewer = viewer;
        addAllToPatterns(readPatternsFromPrefs());
    }

    /**
     * @see org.eclipse.jface.viewers.ViewerFilter#select(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public boolean select(final Viewer viewer, final Object parentElement, final Object element) {
        final String className = ((StackTraceElement)element).getClassName();
        return super.select(viewer, parentElement, className);
    }

    /**
     * Reads the patters to be filtered out of the preferences.
     * 
     * @return The sorted set of filter patterns.
     */
    private final SortedSet readPatternsFromPrefs() {
        final String encoded = SLeakPlugin.getDefault()
                                          .getPreferenceStore()
                                          .getString(SLeakPreferenceConstants.FILTER_PATTERNS);
        return RegexPatternSetFieldEditor.decodePatterns(encoded);
    }

    /**
     * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
     */
    public void propertyChange(final PropertyChangeEvent event) {
        if (SLeakPreferenceConstants.FILTER_PATTERNS == event.getProperty()) {
            clearPatterns();
            addAllToPatterns(readPatternsFromPrefs());
            this.viewer.refresh();
        }
    }
}

/* ########## End of File ########## */